# stage_commit.py
#
# Copyright (C) 2021 Tim Marston <tim@ed.am>
#
# This file is part of stdhome (hereafter referred to as "this program").
# See http://ed.am/dev/stdhome for more information.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import sys, re, getopt, os
from .command import Command
import stdhome.the as the
from stdhome.deployment import Deployment
from stdhome.walker.copy_in import CopyInWalker
from stdhome.walker.walker import Walker


class StageCommitCommand( Command ):


	def print_help( self ):
		print("Usage: " + the.program.name + " stage-commit [--repo=REPO]")
		print()
		#      01234567890123456789012345678901234567890123456789012345678901234567890123456789
		print("Commit staged changes to the local repository to the remote repository.")
		print()
		print("In addition to using the primary commands (such as add and remove) to modify a")
		print("remote repository, you can also set up changes in the local repository and")
		print("manually commit them to a remote repository when ready.  This can be done with")
		print("the staging commands.  Note, some primary commands will not work when there are")
		print("staged changes.  You can revert all staged changes with stage-revert.")
		print()
		print("This command performs a commit operation of any staged changes, writing them to")
		print("the remote repository.")
		print()
		print("Options:")
		print("  -r, --repo=REPO  select the repo to check-out or create (defaults to 'home')")
		print("  -v, --verbose    display information about what is being done")
		print("      --help       display help and exit")
		exit( 0 )


	def parse_command_line( self ):
		opts, args = getopt.gnu_getopt(
			sys.argv[ 1: ], "r:v",
			[ "repo=", "verbose", "help" ] )
		for opt, optarg in opts:
			if opt in [ '--repo', '-r' ]:
				if not re.search( '^[-a-zA-z0-9.]+$', optarg ):
					raise the.program.FatalError(
						'invalid repository name: ' + optarg )
				the.repo = optarg
			elif opt in [ '--verbose', '-v' ]:
				the.verbose += 1
			elif opt == "--help":
				self.print_help()

		# discard first argument (the command)
		args.pop( 0 )

		# check remaining arguments
		if len( args ) != 0:
			raise the.program.UsageError( 'too many arguments' )


	def run( self ):

		# set up repo and check it exists
		the.repo.check_dir_exists()

		# initialise deployment and ensure it's not ongoing
		deployment = Deployment()
		deployment.check_ongoing( False )

		# check for local changes
		if not the.repo.vcs.has_changes():
			raise the.program.FatalError( 'nothing staged' )
            
		# commit repo
		if the.verbose >= 1: print("committing " + the.repo.dir)
		the.repo.vcs.commit()
